'use strict'

const Axios = require('axios')
const axios = Axios.create({
  baseURL: 'https://api.current-rms.com/api/v1/'
})

/**
 * Base Class.
 * @class
 * @name Client
 * @param {string} apiKey - Api Key
 * @param {string} subdomain - Subdomain
 */
function Client (apiKey, subdomain) {
  if (apiKey && subdomain) {
    axios.defaults.headers.common['X-AUTH-TOKEN'] = apiKey
    axios.defaults.headers.common['X-SUBDOMAIN'] = subdomain
  } else {
    throw new Error('No credentials provided for Current RMS API.')
  }
}

/**
 * GET Accessory.
 * @memberof Client
 * @method getAccessory
 * @param {number} productId - Product ID
 * @param {number} accessoryId - Accessory ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getAccessory = (productId, accessoryId) => {
  return axios.get(`products/${productId}/accessories/${accessoryId}`)
}

/**
 * PUT Accessory.
 * @param {number} productId - Product ID
 * @param {number} accessoryId - Accessory ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putAccessory = (productId, accessoryId, body) => {
  return axios.put(`products/${productId}/accessories/${accessoryId}`, body)
}

/**
 * GET List of Accessories.
 * @param {number} productId - Product ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getAccessories = (productId) => {
  return axios.get(`products/${productId}/accessories`)
}

/**
 * POST Accessory.
 * @param {number} productId - Product ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postAccessories = (productId, body) => {
  return axios.post(`products/${productId}/accessories`, body)
}

/**
 * GET Action.
 * @param {number} actionId - Action ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getAction = (actionId) => {
  return axios.get(`actions/${actionId}`)
}

/**
 * GET Actions.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getActions = (queries) => {
  return axios.get(`actions${addQueryParameters(queries)}`)
}

/**
 * GET Activity.
 * @param {number} activityId - Activity ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getActivity = (activityId) => {
  return axios.get(`activities/${activityId}`)
}

/**
 * PUT Activity.
 * @param {number} activityId - Activity ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putActivity = (activityId, body) => {
  return axios.put(`activities/${activityId}`, body)
}

/**
 * DELETE Activity.
 * @param {number} activityId - Activity ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.deleteActivity = (activityId) => {
  return axios.delete(`activities/${activityId}`)
}

/**
 * GET List of Activities.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getActivities = (queries) => {
  return axios.get(`activities${addQueryParameters(queries)}`)
}

/**
 * POST Activities.
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postActivities = (body) => {
  return axios.post('activities', body)
}

/**
 * GET Attachment.
 * @param {number} attachmentId - Attachment ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getAttachment = (attachmentId) => {
  return axios.get(`attachments/${attachmentId}`)
}

/**
 * PUT Attachment.
 * @param {number} attachmentId - Attachment ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putAttachment = (attachmentId, body) => {
  return axios.put(`attachments/${attachmentId}`, body)
}

/**
 * DELETE Attachment.
 * @param {number} attachmentId - Attachment ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.deleteAttachment = (attachmentId) => {
  return axios.delete(`attachments/${attachmentId}`)
}

/**
 * GET List of Attachments.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getAttachments = (queries) => {
  return axios.get(`attachments${addQueryParameters(queries)}`)
}

/**
 * POST Attachments.
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postAttachments = (body) => {
  return axios.post('attachments', body)
}

/**
 * GET Comment.
 * @param {number} discussionId - Discussion ID
 * @param {number} commentId - Comment ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getComment = (discussionId, commentId) => {
  return axios.get(`discussions/${discussionId}/comments/${commentId}`)
}

/**
 * PUT Comment.
 * @param {number} discussionId - Discussion ID
 * @param {number} commentId - Comment ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putComment = (discussionId, commentId, body) => {
  return axios.put(`discussions/${discussionId}/comments/${commentId}`, body)
}

/**
 * GET List of Comments.
 * @param {number} discussionId - Discussion ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getComments = (discussionId) => {
  return axios.get(`discussions/${discussionId}/comments`)
}

/**
 * POST Comments.
 * @param {number} discussionId - Discussion ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postComments = (discussionId, body) => {
  return axios.post(`discussions/${discussionId}/comments`, body)
}

/**
 * GET Country.
 * @param {number} countryId - Country ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getCountry = (countryId) => {
  return axios.get(`countries/${countryId}`)
}

/**
 * GET List of Countries.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getCountries = (queries) => {
  return axios.get(`countries${addQueryParameters(queries)}`)
}

/**
 * GET Custom Field Group.
 * @param {number} customFieldGroupId - Custom Field Group ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getCustomFieldGroup = (customFieldGroupId) => {
  return axios.get(`custom_field_groups/${customFieldGroupId}`)
}

/**
 * PUT Custom Field Group.
 * @param {number} customFieldGroupId - Custom Field Group ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putCustomFieldGroup = (customFieldGroupId, body) => {
  return axios.put(`custom_field_groups/${customFieldGroupId}`, body)
}

/**
 * GET List of Custom Field Groups.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getCustomFieldGroups = (queries) => {
  return axios.get(`custom_field_groups${addQueryParameters(queries)}`)
}

/**
 * POST Custom Field Groups.
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postCustomFieldGroups = (body) => {
  return axios.post(body)
}

/**
 * GET Custom Field.
 * @param {number} customFieldId - Custom Field ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getCustomField = (customFieldId) => {
  return axios.get(`custom_fields/${customFieldId}`)
}

/**
 * PUT Custom Field.
 * @param {number} customFieldId - Custom Field ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putCustomField = (customFieldId, body) => {
  return axios.put(`custom_fields/${customFieldId}`, body)
}

/**
 * GET List of Custom Fields.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getCustomFields = (queries) => {
  return axios.get(`custom_fields${addQueryParameters(queries)}`)
}

/**
 * POST Custom Fields.
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postCustomFields = (body) => {
  return axios.post(`custom_fields`, body)
}

/**
 * GET Discount Category.
 * @param {number} discountCategoryId - Discount Category ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getDiscountCategory = (discountCategoryId) => {
  return axios.get(`discount_categories/${discountCategoryId}`)
}

/**
 * Get Discount Categories.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getDiscountCategories = (queries) => {
  return axios.get(`discount_categories${addQueryParameters(queries)}`)
}

/**
 * GET Discussion.
 * @param {number} discussionId - Discussion ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getDiscussion = (discussionId) => {
  return axios.get(`discussions/${discussionId}`)
}

/**
 * PUT Discussion.
 * @param {number} discussionId - Discussion ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putDiscussion = (discussionId, body) => {
  return axios.put(`discussions/${discussionId}`, body)
}

/**
 * DELETE Discussion.
 * @param {number} discussionId - Discussion ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.deleteDiscussion = (discussionId) => {
  return axios.delete(`discussions/${discussionId}`)
}

/**
 * GET List of Discussions.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getDiscussions = (queries) => {
  return axios.get(`discussions${addQueryParameters(queries)}`)
}

/**
 * POST Discussions.
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postDiscussions = (body) => {
  return axios.post(`discussions`, body)
}

/**
 * GET Document.
 * @param {number} documentId - document id
 * @return {Promise} Promise from Axios
 */
Client.prototype.getDocument = (discussionId) => {
  return axios.get(`documents/${discussionId}`)
}

/**
 * GET List of Documents.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getDocuments = (queries) => {
  return axios.get(`documents${addQueryParameters(queries)}`)
}

/**
 * GET Invoice.
 * @param {number} invoiceId - Invoice ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getInvoice = (invoiceId) => {
  return axios.get(`invoices/${invoiceId}`)
}

/**
 * GET List of Invoices.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getInvoices = (queries) => {
  return axios.get(`invoices${addQueryParameters(queries)}`)
}

/**
 * GET Prepare Invoice Document.
 * @param {number} invoiceId - Invoice ID
 * @param {number} invoiceDocumentId - Invoice Document ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getPrepareInvoiceDocument = (invoiceId, invoiceDocumentId) => {
  return axios.get(`invoices/${invoiceId}/prepare_document?document_id=${invoiceDocumentId}`)
}

/**
 * GET Invoice Item.
 * @param {number} invoiceId - Invoice ID
 * @param {number} invoiceItemId - Invoice Item ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getInvoiceItem = (invoiceId, invoiceItemId) => {
  return axios.get(`invoices/${invoiceId}/invoice_items/${invoiceItemId}`)
}

/**
 * GET List of Invoice Items.
 * @param {number} invoiceId - Invoice ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getInvoiceItems = (invoiceId) => {
  return axios.get(`invoices/${invoiceId}/invoice_items`)
}

/**
 * GET Invoice Document.
 * @param {number} invoiceDocumentId - Invoice Document ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getInvoiceDocument = (invoiceDocumentId) => {
  return axios.get(`invoice_documents/${invoiceDocumentId}`)
}

/**
 * GET Invoice Document PDF.
 * @param {number} invoiceDocumentId - Invoice Document ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getInvoiceDocumentPdf = (invoiceDocumentId) => {
  return axios.get(`invoice_documents/${invoiceDocumentId}.pdf`, {
    responseType: 'arraybuffer'
  })
}

/**
 * GET List of Invoice Documents.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getInvoiceDocuments = (queries) => {
  return axios.get(`invoice_documents${addQueryParameters(queries)}`)
}

/**
 * GET Member Transaction.
 * @param {number} invoiceId - Invoice ID
 * @param {number} memberTransactionId - Member Transaction ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getMemberTransaction = (invoiceId, memberTransactionId) => {
  return axios.get(`invoices/${invoiceId}/member_transactions/${memberTransactionId}`)
}

/**
 * GET List of Member Transactions.
 * @param {number} invoiceId - Invoice ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getMemberTransactions = (invoiceId) => {
  return axios.get(`invoices/${invoiceId}/member_transactions`)
}

/**
 * GET List Name.
 * @param {number} listNameId - List Name ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getListName = (listNameId) => {
  return axios.get(`list_names/${listNameId}`)
}

/**
 * PUT List Name.
 * @param {number} listNameId - List Name ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putListName = (listNameId, body) => {
  return axios.put(`list_names/${listNameId}`, body)
}

/**
 * DELETE List Name.
 * @param {number} listNameId - List Name ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.deleteListName = (listNameId) => {
  return axios.delete(`list_names/${listNameId}`)
}

/**
 * GET List of List Names.
 * @return {Promise} Promise from Axios
 */
Client.prototype.getListNames = () => {
  return axios.get(`list_names`)
}

/**
 * POST List Names.
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postListNames = (body) => {
  return axios.post(`list_names`, body)
}

/**
 * GET Member.
 * @param {number} memberId - Member ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getMember = (memberId) => {
  return axios.get(`members/${memberId}`)
}

/**
 * PUT Member.
 * @param {number} memberId - Member ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putMember = (memberId, body) => {
  return axios.put(`members/${memberId}`, body)
}

/**
 * DELETE Member.
 * @param {number} memberId - Member ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.deleteMember = (memberId) => {
  return axios.delete(`members/${memberId}`)
}

/**
 * POST Member.
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postMember = (body) => {
  return axios.post(`members`, body)
}

/**
 * GET List of Members.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getMembers = (queries) => {
  return axios.get(`members${addQueryParameters(queries)}`)
}

/**
 * GET Prepare Member Document.
 * @param {number} memberId - Member ID
 * @param {number} memberDocumentId - member document id
 * @return {Promise} Promise from Axios
 */
Client.prototype.getPrepareMemberDocument = (memberId, memberDocumentId) => {
  return axios.get(`members/${memberId}/prepare_document`, {
    params: {
      document_id: memberDocumentId
    }
  })
}

/**
 * GET Prepare Member Document PDF.
 * @param {number} memberId - Member ID
 * @param {number} memberDocumentId - member document id
 * @return {Promise} Promise from Axios
 */
Client.prototype.getPrepareMemberDocumentPdf = (memberId, memberDocumentId) => {
  return axios.get(`members/${memberId}/prepare_document.pdf`, {
    responseType: 'arraybuffer',
    params: {
      document_id: memberDocumentId
    }
  })
}

/**
 * GET Opportunity.
 * @param {number} opportunityId - Opportunity ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunity = (opportunityId) => {
  return axios.get(`opportunities/${opportunityId}`)
}

/**
 * PUT Opportunity.
 * @param {number} opportunityId - Opportunity ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putOpportunity = (opportunityId, body) => {
  return axios.put(`opportunities/${opportunityId}`, body)
}

/**
 * DELETE Opportunity.
 * @param {number} opportunityId - Opportunity ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.deleteOpportunity = (opportunityId) => {
  return axios.delete(`opportunities/${opportunityId}`)
}

/**
 * POST Opportunity.
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postOpportunity = (body) => {
  return axios.post(`opportunities`, body)
}

/**
 * GET List of Opportunities
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunities = (queries) => {
  return axios.get(`opportunities${addQueryParameters(queries)}`)
}

/**
 * GET Prepare Opportunity Document.
 * @param {number} opportunityId - Opportunity ID
 * @param {number} opportunityDocumentId - Opportunity Document ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getPrepareOpportunityDocument = (opportunityId, opportunityDocumentId) => {
  return axios.get(`opportunities/${opportunityId}/prepare_document`, {
    params: {
      document_id: opportunityDocumentId
    }
  })
}

/**
 * POST Opportunity Checkout.
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postOpportunityCheckout = (body) => {
  return axios.post(`opportunities/checkout`, body)
}

/**
 * POST Opportunity Quick Allocate.
 * @param {number} opportunityId - Opportunity ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postOpportunityQuickAllocate = (opportunityId, body) => {
  return axios.post(`opportunities/${opportunityId}/quick_allocate`, body)
}

/**
 * POST Opportunity Quick Prepare.
 * @param {number} opportunityId - Opportunity ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postOpportunityQuickPrepare = (opportunityId, body) => {
  return axios.post(`opportunities/${opportunityId}/quick_prepare`, body)
}

/**
 * POST Opportunity Quick Book Out.
 * @param {number} opportunityId - Opportunity ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postOpportunityQuickBookOut = (opportunityId, body) => {
  return axios.post(`opportunities/${opportunityId}/quick_book_out`, body)
}

/**
 * POST Opportunity Quick Check In.
 * @param {number} opportunityId - Opportunity ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postOpportunityQuickCheckIn = (opportunityId, body) => {
  return axios.post(`opportunities/${opportunityId}/quick_check_in`, body)
}

/**
 * POST Opportunity Prepare.
 * @param {number} opportunityId - Opportunity ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postOpportunityPrepare = (opportunityId, body) => {
  return axios.post(`opportunities/${opportunityId}/quick_prepare`, body)
}

/**
 * POST Opportunity Set Container.
 * @param {number} opportunityId - Opportunity ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postOpportunitySetContainer = (opportunityId, body) => {
  return axios.post(`opportunities/${opportunityId}/set_container`, body)
}

/**
 * POST Opportunity Confirm.
 * @param {number} opportunityId - Opportunity ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postOpportunityConfirm = (opportunityId, body) => {
  return axios.post(`opportunities/${opportunityId}/confirm`, body)
}

/**
 * POST Opportunity Book Out.
 * @param {number} opportunityId - Opportunity ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postOpportunityBookOut = (opportunityId, body) => {
  return axios.post(`opportunities/${opportunityId}/book_out`, body)
}

/**
 * POST Opportunity Check In.
 * @param {number} opportunityId - Opportunity ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postOpportunityCheckIn = (opportunityId, body) => {
  return axios.post(`opportunities/${opportunityId}/check_in`, body)
}

/**
 * POST Opportunity Revert Status.
 * @param {number} opportunityId - Opportunity ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postOpportunityRevertStatus = (opportunityId, body) => {
  return axios.post(`opportunities/${opportunityId}/revert_status`, body)
}

/**
 * POST Opportunity Finalise Status.
 * @param {number} opportunityId - Opportunity ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postOpportunityFinaliseStatus = (opportunityId, body) => {
  return axios.post(`opportunities/${opportunityId}/finalise_services`, body)
}

/**
 * GET Opportunity Convert To Quote.
 * @param {number} opportunityId - Opportunity ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunityConvertToQuote = (opportunityId) => {
  return axios.get(`opportunities/${opportunityId}/convert_to_quote`)
}

/**
 * GET Opportunity Convert To Order.
 * @param {number} opportunityId - Opportunity ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunityConvertToOrder = (opportunityId) => {
  return axios.get(`opportunities/${opportunityId}/convert_to_order`)
}

/**
 * GET Opportunity Revert To Quote.
 * @param {number} opportunityId - Opportunity ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunityRevertToQuote = (opportunityId) => {
  return axios.get(`opportunities/${opportunityId}/revert_to_quote`)
}

/**
 * GET Opportunity Mark As Provisional.
 * @param {number} opportunityId - Opportunity ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunityMarkAsProvisional = (opportunityId) => {
  return axios.get(`opportunities/${opportunityId}/mark_as_provisional`)
}

/**
 * GET Opportunity Mark As Reserved.
 * @param {number} opportunityId - Opportunity ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunityMarkAsReserved = (opportunityId) => {
  return axios.get(`opportunities/${opportunityId}/mark_as_reserved`)
}

/**
 * GET Opportunity Cancel.
 * @param {number} opportunityId - Opportunity ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunityCancel = (opportunityId) => {
  return axios.get(`opportunities/${opportunityId}/cancel`)
}

/**
 * POST Opportunity Finalise Check In.
 * @param {number} opportunityId - Opportunity ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postOpportunityFinaliseCheckIn = (opportunityId, body) => {
  return axios.post(`opportunities/${opportunityId}/finalise`, body)
}

/**
 * GET Opportunity Cost.
 * @param {number} opportunityId - Opportunity ID
 * @param {number} opportunityCostId - Opportunity Cost ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunityCost = (opportunityId, opportunityCostId) => {
  return axios.get(`opportunities/${opportunityId}/opportunity_costs/${opportunityCostId}`)
}

/**
 * GET List of Opportunity Costs.
 * @param {number} opportunityId - Opportunity ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunityCosts = (opportunityId) => {
  return axios.get(`opportunities/${opportunityId}/opportunity_costs`)
}

/**
 * GET Opportunity Item.
 * @param {number} opportunityId - Opportunity ID
 * @param {number} opportunityItemId - Opportunity Item ID
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunityItem = (opportunityId, opportunityItemId, queries) => {
  return axios.get(`opportunities/${opportunityId}/opportunity_items/${opportunityItemId}${addQueryParameters(queries)}`)
}

/**
 * DELETE Opportunity Item.
 * @param {number} opportunityId - Opportunity ID
 * @param {number} opportunityItemId - Opportunity Item ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.deleteOpportunityItem = (opportunityId, opportunityItemId) => {
  return axios.delete(`opportunities/${opportunityId}/opportunity_items/${opportunityItemId}`)
}

/**
 * GET List of Opportunity Items.
 * @param {number} opportunityId - Opportunity ID
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunityItems = (opportunityId, queries) => {
  return axios.get(`opportunities/${opportunityId}/opportunity_items${addQueryParameters(queries)}`)
}

/**
 * POST Opportunity Item Allocate Resource.
 * @param {number} opportunityId - Opportunity ID
 * @param {number} opportunityItemId - Opportunity Item ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postOpportunityItemAllocateResource = (opportunityId, opportunityItemId, body) => {
  return axios.post(`opportunities/${opportunityId}/opportunity_items/${opportunityItemId}/allocate_resource`, body)
}

/**
 * POST Opportunity Item Deallocate Resource.
 * @param {number} opportunityId - Opportunity ID
 * @param {number} opportunityItemId - Opportunity Item ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postOpportunityItemDeallocateResource = (opportunityId, opportunityItemId, body) => {
  return axios.post(`opportunities/${opportunityId}/opportunity_items/${opportunityItemId}/deallocate_resource`, body)
}

/**
 * GET Opportunity Item Asset.
 * @param {number} opportunityId - Opportunity ID
 * @param {number} opportunityItemId - Opportunity Item ID
 * @param {number} opportunityItemAssetId - Opportunity Item Asset ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunityItemAsset = (opportunityId, opportunityItemId, opportunityItemAssetId) => {
  return axios.get(`opportunities/${opportunityId}/opportunity_items/${opportunityItemId}/opportunity_item_assets/${opportunityItemAssetId}`)
}

/**
 * GET List of Opportunity Item Assets.
 * @param {number} opportunityId - Opportunity ID
 * @param {number} opportunityItemId - Opportunity Item ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunityItemAssets = (opportunityId, opportunityItemId) => {
  return axios.get(`opportunities/${opportunityId}/opportunity_items/${opportunityItemId}/opportunity_item_assets`)
}

/**
 * GET Opportunity Document.
 * @param {number} opportunityDocumentId - Opportunity Document ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunityDocument = (opportunityDocumentId) => {
  return axios.get(`opportunity_documents/${opportunityDocumentId}`)
}

/**
 * GET Opportunity Document PDF.
 * @param {number} opportunityDocumentId - Opportunity Document ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunityDocumentPdf = (opportunityDocumentId) => {
  return axios.get(`opportunity_documents/${opportunityDocumentId}.pdf`, {
    responseType: 'arraybuffer'
  })
}

/**
 * GET Opportunity Documents.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunityDocuments = (queries) => {
  return axios.get(`opportunity_documents${addQueryParameters(queries)}`)
}

/**
 * PUT Opportunity Document.
 * @param {number} opportunityDocumentId - Opportunity Document ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putOpportunityDocument = (opportunityDocumentId, body) => {
  return axios.put(`opportunity_documents/${opportunityDocumentId}`, body)
}

/**
 * GET Product Group.
 * @param {number} productGroupId - Product Group ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getProductGroup = (productGroupId) => {
  return axios.get(`product_groups/${productGroupId}`)
}

/**
 * PUT Product Group.
 * @param {number} productGroupId - Product Group ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putProductGroup = (productGroupId, body) => {
  return axios.put(`product_groups/${productGroupId}`, body)
}

/**
 * POST Product Group.
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postProductGroup = (body) => {
  return axios.post(`product_groups`, body)
}

/**
 * GET List of Product Groups.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getProductGroups = (queries) => {
  return axios.get(`product_groups${addQueryParameters(queries)}`)
}

/**
 * GET Product.
 * @param {number} productId - Product ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getProduct = (productId) => {
  return axios.get(`products/${productId}`)
}

/**
 * PUT Product.
 * @param {number} productId - Product ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putProduct = (productId, body) => {
  return axios.put(`products/${productId}`, body)
}

/**
 * POST Product.
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postProduct = (body) => {
  return axios.post(`products`, body)
}

/**
 * GET List of Products.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getProducts = (queries) => {
  return axios.get(`products${addQueryParameters(queries)}`)
}

/**
 * GET Product Inventory.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getProductInventory = (queries) => {
  return axios.get(`products/inventory${addQueryParameters(queries)}`)
}

/**
 * GET Prepare Product Document.
 * @param {number} productId - Product ID
 * @param {number} productDocumentId - Product Document ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getPrepareProductDocument = (productId, productDocumentId) => {
  return axios.get(`products/${productId}/prepare_document`, {
    params: {
      document_id: productDocumentId
    }
  })
}

/**
 * GET Prepare Product Document PDF.
 * @param {number} productId - Product ID
 * @param {number} productDocumentId - Product Document ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunityDocumentPdf = (productId, productDocumentId) => {
  return axios.get(`products/${productId}/prepare_document.pdf`, {
    responseType: 'arraybuffer',
    params: {
      document_id: productDocumentId
    }
  })
}

/**
 * POST Action Group.
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postActionGroup = (body) => {
  return axios.post(`availability/group`, body)
}

/**
 * POST Action Product.
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postActionProduct = (body) => {
  return axios.post(`availability/product`, body)
}

/**
 * GET Product Inventory.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getProductInventory = (queries) => {
  return axios.get(`product_inventories${addQueryParameters(queries)}`)
}

/**
 * GET Product Revenue.
 * @param {number} productId - Product ID
 * @param {number} productRevenueId - Product Revenue ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getProductRevenue = (productId, productRevenueId) => {
  return axios.get(`products/${productId}/product_revenues/${productRevenueId}`)
}

/**
 * GET List of Product Revenues.
 * @param {number} productId - Product ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getProductsRevenues = (productId) => {
  return axios.get(`products/${productId}/product_revenues`)
}

/**
 * GET Project.
 * @param {number} projectId - Project ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getProject = (projectId) => {
  return axios.get(`projects/${projectId}`)
}

/**
 * PUT Project.
 * @param {number} projectId - Project ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putProject = (projectId, body) => {
  return axios.put(`projects/${projectId}`, body)
}

/**
 * DELETE Project.
 * @param {number} projectId - Project ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.deleteProject = (projectId) => {
  return axios.delete(`projects/${projectId}`)
}

/**
 * POST Project.
 * @param {number} projectId - Project ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postProject = (projectId, body) => {
  return axios.post(`projects/${projectId}`, body)
}

/**
 * GET List of Projects.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getProjects = (queries) => {
  return axios.get(`projects${addQueryParameters(queries)}`)
}

/**
 * GET Prepare Project Document.
 * @param {number} projectId - Project ID
 * @param {number} projectDocumentId - Project Document ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getPrepareProjectDocument = (projectId, projectDocumentId) => {
  return axios.get(`projects/${projectId}/prepare_document`, {
    params: {
      document_id: projectDocumentId
    }
  })
}

/**
 * GET Project Document.
 * @param {number} projectDocumentId - Project Document ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getProjectDocument = (projectDocumentId) => {
  return axios.get(`project_documents/${projectDocumentId}`)
}

/**
 * GET Project Document PDF.
 * @param {number} projectDocumentId - Project Document ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getProjectDocumentPdf = (projectDocumentId) => {
  return axios.get(`project_documents/${projectDocumentId}.pdf`, {
    responseType: 'arraybuffer'
  })
}

/**
 * GET Project Documents.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getProjectDocuments = (queries) => {
  return axios.get(`project_documents${addQueryParameters(queries)}`)
}

/**
 * GET Quarantine.
 * @param {number} quarantineId - Quarantine ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getQuarantine = (quarantineId) => {
  return axios.get(`quarantines/${quarantineId}`)
}

/**
 * PUT Quarantine.
 * @param {number} quarantineId - Quarantine ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putQuarantine = (quarantineId, body) => {
  return axios.put(`quarantines/${quarantineId}`, body)
}

/**
 * POST Quarantine.
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postQuarantine = (body) => {
  return axios.post(`quarantines`, body)
}

/**
 * POST Quarantine Completion.
 * @param {number} quarantineId - Quarantine ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postQuarantineCompletion = (quarantineId, body) => {
  return axios.post(`quarantines/${quarantineId}/complete`, body)
}

/**
 * GET List of Quarantines.
 * @return {Promise} Promise from Axios
 */
Client.prototype.getQuarantines = () => {
  return axios.get(`quarantines`)
}

/**
 * GET Prepare Quarantine Document.
 * @param {number} quarantineId - Quarantine ID
 * @param {number} quarantineDocumentId - Quarantine Document ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getPrepareQuarantineDocument = (quarantineId, quarantineDocumentId) => {
  return axios.get(`quarantines/${quarantineId}/prepare_document`, {
    params: {
      document_id: quarantineDocumentId
    }
  })
}

/**
 * GET Quarantine Document PDF.
 * @param {number} quarantineId - Quarantine ID
 * @param {number} quarantineDocumentId - Quarantine Document ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getQuarantineDocumentPdf = (quarantineId, quarantineDocumentId) => {
  return axios.get(`quarantines/${quarantineId}.pdf`, {
    responseType: 'arraybuffer',
    params: {
      document_id: quarantineDocumentId
    }
  })
}

/**
 * GET Rate.
 * @param {number} productId - Product ID
 * @param {number} rateId - Rate ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getRate = (productId, rateId) => {
  return axios.get(`products/${productId}/rates/${rateId}`)
}

/**
 * PUT Rate.
 * @param {number} productId - Product ID
 * @param {number} rateId - Rate ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putRate = (productId, rateId, body) => {
  return axios.put(`products/${productId}/rates/${rateId}`, body)
}

/**
 * POST Rate.
 * @param {number} productId - Product ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postRate = (productId, body) => {
  return axios.post(`products/${productId}`, body)
}

/**
 * GET List of Rates.
 * @param {number} productId - Product ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getRates = (productId) => {
  return axios.get(`products/${productId}/rates`)
}

/**
 * GET Rate Definition.
 * @param {number} rateDefinitionId - Rate Definition ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getRateDefinition = (rateDefinitionId) => {
  return axios.get(`rate_definitions/${rateDefinitionId}`)
}

/**
 * GET List of Rate Definitions.
 * @return {Promise} Promise from Axios
 */
Client.prototype.getRateDefinitions = () => {
  return axios.get(`rate_definitions`)
}

/**
 * GET Service.
 * @param {number} serviceId - Service ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getService = (serviceId) => {
  return axios.get(`services/${serviceId}`)
}

/**
 * PUT Service.
 * @param {number} serviceId - Service ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putService = (serviceId, body) => {
  return axios.put(`services/${serviceId}`, body)
}

/**
 * POST Service.
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postService = (body) => {
  return axios.post(`services`, body)
}

/**
 * GET List of Services.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getServices = (queries) => {
  return axios.get(`services${addQueryParameters(queries)}`)
}

/**
 * GET Stock Booking.
 * @param {number} stockBookingId - Stock Booking ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getStockBooking = (stockBookingId) => {
  return axios.get(`stock_bookings/${stockBookingId}`)
}

/**
 * GET List of Stock Bookings.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getStockBookings = (queries) => {
  return axios.get(`stock_bookings${addQueryParameters(queries)}`)
}

/**
 * GET Stock Level.
 * @param {number} productId - Product ID
 * @param {number} stockLevelId - Stock Level ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getStockLevel = (productId, stockLevelId) => {
  return axios.get(`products/${productId}/stock_levels/${stockLevelId}`)
}

/**
 * PUT Stock Level.
 * @param {number} productId - Product ID
 * @param {number} stockLevelId - Stock Level ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putStockLevel = (productId, stockLevelId, body) => {
  return axios.put(`products/${productId}/stock_levels/${stockLevelId}`, body)
}

/**
 * POST Stock Level.
 * @param {number} productId - Product ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postStockLevel = (productId, body) => {
  return axios.post(`products/${productId}/stock_levels`, body)
}

/**
 * GET List of Stock Levels.
 * @param {number} productId - Product ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getStockLevels = (productId) => {
  return axios.get(`products/${productId}/stock_levels`)
}

/**
 * GET Service Stock Level.
 * @param {number} serviceId - Service ID
 * @param {number} stockLevelId - Stock Level ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getServiceStockLevel = (serviceId, stockLevelId) => {
  return axios.get(`services/${serviceId}/stock_levels/${stockLevelId}`)
}

/**
 * PUT Service Stock Level.
 * @param {number} serviceId - Service ID
 * @param {number} stockLevelId - Stock Level ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putServiceStockLevel = (serviceId, stockLevelId, body) => {
  return axios.put(`services/${serviceId}/stock_levels/${stockLevelId}`, body)
}

/**
 * POST Service Stock Level.
 * @param {number} serviceId - Service ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postServiceStockLevel = (serviceId, body) => {
  return axios.post(`services/${serviceId}/stock_levels`, body)
}

/**
 * GET List of Service Stock Levels.
 * @param {number} serviceId - Service ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getServiceStockLevels = (serviceId) => {
  return axios.get(`services/${serviceId}/stock_levels`)
}

/**
 * GET Stock Transaction.
 * @param {number} productId - Product ID
 * @param {number} stockLevelId - Stock Level ID
 * @param {number} stockTransactionId - Stock Transaction ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getStockTransaction = (productId, stockLevelId, stockTransactionId) => {
  return axios.get(`products/${productId}/stock_levels/${stockLevelId}/stock_transactions/${stockTransactionId}`)
}

/**
 * PUT Stock Transaction.
 * @param {number} productId - Product ID
 * @param {number} stockLevelId - Stock Level ID
 * @param {number} stockTransactionId - Stock Transaction ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putStockTransaction = (productId, stockLevelId, stockTransactionId, body) => {
  return axios.put(`products/${productId}/stock_levels/${stockLevelId}/stock_transactions/${stockTransactionId}`, body)
}

/**
 * POST Stock Transaction.
 * @param {number} productId - Product ID
 * @param {number} stockLevelId - Stock Level ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postStockTransaction = (productId, stockLevelId, body) => {
  return axios.post(`products/${productId}/stock_levels/${stockLevelId}/stock_transactions`, body)
}

/**
 * GET List of Stock Transactions.
 * @param {number} productId - Product ID
 * @param {number} stockLevelId - Stock Level ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getStockTransactions = (productId, stockLevelId) => {
  return axios.get(`products/${productId}/stock_levels/${stockLevelId}/stock_transactions`)
}

/**
 * POST Store Transfer.
 * @param {number} productId - Product ID
 * @param {number} stockLevelId - Stock Level ID
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postStoreTransfer = (productId, stockLevelId, body) => {
  return axios.post(`products/${productId}/stock_levels/${stockLevelId}/store_transfer`, body)
}

/**
 * GET Store.
 * @param {number} storeId - Store ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getStore = (storeId) => {
  return axios.get(`stores/${storeId}`)
}

/**
 * PUT Store.
 * @param {number} storeId - Store ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putStore = (storeId, body) => {
  return axios.put(`stores/${storeId}`, body)
}

/**
 * POST Store.
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postStore = (body) => {
  return axios.post(`stores`, body)
}

/**
 * GET List of Stores.
 * @return {Promise} Promise from Axios
 */
Client.prototype.getStores = () => {
  return axios.get(`stores`)
}

/**
 * GET Vehicle.
 * @param {number} vehicleId - Vehicle ID
 * @return {Promise} Promise from Axios
 */
Client.prototype.getVehicle = (vehicleId) => {
  return axios.get(`vehicles/${vehicleId}`)
}

/**
 * PUT Vehicle.
 * @param {number} vehicleId - Vehicle ID
 * @param {Object} body - PUT Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.putVehicle = (vehicleId, body) => {
  return axios.put(`vehicles/${vehicleId}`, body)
}

/**
 * POST Vehicle.
 * @param {Object} body - POST Body
 * @return {Promise} Promise from Axios
 */
Client.prototype.postVehicle = (body) => {
  return axios.post(`vehicles`, body)
}

/**
 * GET List of Vehicles.
 * @return {Promise} Promise from Axios
 */
Client.prototype.getVehicles = () => {
  return axios.get(`vehicles`)
}

/**
 * GET Activity Tag Cloud.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getActivityTagCloud = (queries) => {
  return axios.get(`activities/tag_cloud${addQueryParameters(queries)}`)
}

/**
 * GET Invoice Tag Cloud.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getInvoiceTagCloud = (queries) => {
  return axios.get(`invoices/tag_cloud${addQueryParameters(queries)}`)
}

/**
 * GET Member Tag Cloud.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getMemberTagCloud = (queries) => {
  return axios.get(`members/tag_cloud${addQueryParameters(queries)}`)
}

/**
 * GET Opportunities Tag Cloud.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getOpportunitiesTagCloud = (queries) => {
  return axios.get(`opportunities/tag_cloud${addQueryParameters(queries)}`)
}

/**
 * GET Projects Tag Cloud.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getProjectTagCloud = (queries) => {
  return axios.get(`projects/tag_cloud${addQueryParameters(queries)}`)
}

/**
 * GET Product Tag Cloud.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getProductTagCloud = (queries) => {
  return axios.get(`products/tag_cloud${addQueryParameters(queries)}`)
}

/**
 * GET Service Tag Cloud.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {Promise} Promise from Axios
 */
Client.prototype.getServiceTagCloud = (queries) => {
  return axios.get(`services/tag_cloud${addQueryParameters(queries)}`)
}

Client.prototype._all = (requests) => {
  return Promise.all(requests)
}

/**
 * Query String Builder.
 * @param {(string[]|string)} queries - An Array or String of query parameters
 * @return {string} query string
 */
function addQueryParameters (queries) {
  if (queries) {
    if (Array.isArray(queries)) {
      return `?${queries.join('&')}`
    } else if (typeof queries === 'string') {
      return `?${queries}`
    } else {
      throw new TypeError('Query parameters must be an Array or String.')
    }
  } else {
    return ''
  }
}

/**
 * Export endpoints.
 * @module Client
 */
module.exports = Client
